*
* 32mux.asm
* Tom Dickens - 10/27/2002
* Program to talk to the 10x32 MUX board.
* There are 8 data lines and 2 address lines to the MUX board.
*    Data lines are B0-B7
*    Address lines are A6 and A7
* This test program will do a binary count on the 32-bit output.
*
	ORG $B600
	BSR Init32	; this clears address 0-3 

Top:
	BSR Load32 ; put the 32-bits of data to the MUX
	INC $00	; Increment data in address 0
	BNE Top ; if it rolled over to zero, increment next
	INC $01	; Increment data in address 1
	BNE Top ; if it rolled over to zero, increment next
	INC $02	; Increment data in address 2
	BNE Top ; if it rolled over to zero, increment next
	INC $03	; Increment data in address 3
	BRA Top ; goto top

* Initialize the 32-MUX stuff.
* Set the address to 11, and set the address lines to outputs.
Init32:
	PSHB
	LDAB $1026	; get the current value from Port A
	ORAB #%10000000	; set bit 7, data direction for A7
	STAB $1026	; set PACTL
*
	CLR  $1004	; clear output on Port B
	LDAB $1000	; get the current value from Port A
	ANDB #%00111111	; clear bits A6 and A7
	STAB $1000	; set address 0
*
	LDAB #$00
	STAB $00	; set 32-bits in memory to zero.
	STAB $01
	STAB $02
	STAB $03
	BSR Load32 ; put the 32-bits of data to the MUX
*
	PULB
	RTS

* Subroutine to take data from memory addresses 0-3 and 
* put that 32-bits of data to the MUX board. 
* Assumes that the address is set to 00. Will leave it
* in that state.
* Don't load address 0, 1, 2,  in order, since the change
* from 1 to 2 is a 2-bit change and can cause a glitch in
* the 74HC138s output. Instead make sure to use Gray code,
* which has a 1-bit change. Thus we will order the addresses
* 0, 1, 3, 2.
Load32:
	PSHA
	PSHB
	LDAB $1000	; get the current value from Port A
	ANDB #%00111111	; clear bits A6 and A7
* Do 0
	LDAA $00 	; get data from address 0
	STAA $1004	; put data to Port B

* Do 1
	ORAB #%01000000	; set bit A6
	STAB $1000	; set address 1
	LDAA $01 	; get data from address 1
	STAA $1004	; put data to Port B

* Do 3
	ORAB #%11000000	; set bits A6 and A7
	STAB $1000	; set address 3
	LDAA $03 	; get data from address 3
	STAA $1004	; put data to Port B

* Do 2
	ANDB #%00111111	; clear bits A6 and A7
	ORAB #%10000000	; set bit A7
	STAB $1000	; set address 2
	LDAA $02 	; get data from address 2
	STAA $1004	; put data to Port B

* Set back to 0
	ANDB #%00111111	; clear bits A6 and A7
	LDAB #%00000000
	STAB $1000	; set address 0
*
	PULB
	PULA
	RTS
*
